#ifdef WITH_UNITTESTS
#include "gtest/gtest.h"

#include "Build/Tactical/Items.h"

#include "AmmoTypeModel.h"
#include "CalibreModel.h"
#include "DefaultContentManager.h"
#include "DefaultContentManagerUT.h"
#include "GameInstance.h"
#include "MagazineModel.h"
#include "WeaponModels.h"

TEST(Items, weaponsLoading)
{
  DefaultContentManager * cm = createDefaultCMForTesting();
  ASSERT_TRUE(cm != NULL);
  ASSERT_TRUE(cm->loadGameData());
  EXPECT_TRUE(cm->getWeaponByName("MP5K") != NULL);
  EXPECT_TRUE(cm->getWeapon(9 /* MP5K */) != NULL);
  EXPECT_EQ(cm->getWeaponByName("MP5K"), cm->getWeapon(9 /* MP5K */));
  delete cm;
}

TEST(Items, bug120_cawsAmmo)
{
  DefaultContentManager * cm = createDefaultCMForTesting();
  ASSERT_TRUE(cm->loadGameData());

  // test SAP clip parameters
  const MagazineModel* sapClip = cm->getMagazineByName("CLIPCAWS_10_SAP");
  EXPECT_EQ(sapClip->calibre->internalName, std::string("AMMOCAWS"));
  EXPECT_EQ(sapClip->ammoType->internalName, std::string("AMMO_SUPER_AP"));

  // test FLECH clip parameters
  const MagazineModel* flechClip = cm->getMagazineByName("CLIPCAWS_10_FLECH");
  EXPECT_EQ(flechClip->calibre->internalName, std::string("AMMOCAWS"));
  EXPECT_EQ(flechClip->ammoType->internalName, std::string("AMMO_BUCKSHOT"));

  delete cm;
}

TEST(Items, bug120_12gAmmo)
{
  DefaultContentManager * cm = createDefaultCMForTesting();
  ASSERT_TRUE(cm->loadGameData());

  // test SAP clip parameters
  const MagazineModel* clip = cm->getMagazineByName("CLIP12G_7");
  EXPECT_EQ(clip->calibre->internalName,           std::string("AMMO12G"));
  EXPECT_EQ(clip->ammoType->internalName,          std::string("AMMO_REGULAR"));

  // test FLECH clip parameters
  const MagazineModel* clipBuckshot = cm->getMagazineByName("CLIP12G_7_BUCKSHOT");
  EXPECT_EQ(clipBuckshot->calibre->internalName,           std::string("AMMO12G"));
  EXPECT_EQ(clipBuckshot->ammoType->internalName,          std::string("AMMO_BUCKSHOT"));

  delete cm;
}

TEST(Items, bug120_cawsDefaultMag)
{
  DefaultContentManager * cm = createDefaultCMForTesting();
  ASSERT_TRUE(cm->loadGameData());

  GCM = cm;

  const WeaponModel *caws = cm->getWeaponByName("CAWS");
  ASSERT_TRUE(caws != NULL);

  const MagazineModel* flechClip = cm->getMagazineByName("CLIPCAWS_10_FLECH");
  const MagazineModel* sapClip = cm->getMagazineByName("CLIPCAWS_10_SAP");
  ASSERT_TRUE(flechClip != NULL);
  ASSERT_TRUE(sapClip != NULL);

  EXPECT_EQ(DefaultMagazine(caws->getItemIndex()), flechClip->getItemIndex());

  EXPECT_EQ(FindReplacementMagazine(sapClip->calibre, 10, AMMO_BUCKSHOT), flechClip->getItemIndex());
  EXPECT_EQ(FindReplacementMagazine(sapClip->calibre, 10, AMMO_SUPER_AP), sapClip->getItemIndex());

  delete cm;
}

TEST(Items, bug120_spas15DefaultMag)
{
  DefaultContentManager * cm = createDefaultCMForTesting();
  ASSERT_TRUE(cm->loadGameData());

  GCM = cm;

  const WeaponModel *spas15 = cm->getWeaponByName("SPAS15");
  ASSERT_TRUE(spas15 != NULL);

  const MagazineModel* clipBuckshot = cm->getMagazineByName("CLIP12G_7_BUCKSHOT");
  const MagazineModel* clip = cm->getMagazineByName("CLIP12G_7");
  ASSERT_TRUE(clipBuckshot != NULL);
  ASSERT_TRUE(clip != NULL);

  EXPECT_EQ(DefaultMagazine(spas15->getItemIndex()), clipBuckshot->getItemIndex());

  EXPECT_EQ(FindReplacementMagazine(clip->calibre, 7, AMMO_BUCKSHOT), clipBuckshot->getItemIndex());
  EXPECT_EQ(FindReplacementMagazine(clip->calibre, 7, AMMO_REGULAR),  clip->getItemIndex());

  delete cm;
}

#endif
